@@ -5,8 +5,10 @@
 #' @param method character. Method to use for multiple comparison adjustment of tail probabilities.
 #' @param return.only character. Quantities to be returned.
 #'
-#' @details
+#' @description 
+#' Infer graphical structures by multiple testing
 #'
+#' @details
 #' The argument \code{method} allows to adjust the tail probabilities obtained from the null distributions of
 #' the Bayes factors for multiple comparisons. Possible choices are: "holm", "bonferroni", "BH", "BY" and "HC".
 #' Apart from "HC", these are passed onto the R function \code{p.adjust}
@@ -24,7 +26,11 @@
 #'
 #' @author Gwenael G.R. Leday and Ilaria Speranza
 #'
-#' @references Leday, G.G.R. and Richardson, S. (2018). Fast Bayesian inference in large Gaussian graphical models. Submitted.
+#' @references
+#' Drton, M., & Perlman, M. D. (2007). Multiple testing and error control in Gaussian graphical model selection. Statistical Science, 430-449.\cr
+#' Goeman, J. J., & Solari, A. (2014). Multiple hypothesis testing in genomics. Statistics in medicine, 33(11), 1946-1978.\cr
+#' Donoho, D., & Jin, J. (2015). Higher criticism for large-scale inference, especially for rare and weak effects. Statistical Science, 30(1), 1-25.\cr
+#' Klaus, B., & Strimmer, K. (2012). Signal identification for rare and weak features: higher criticism or false discovery rates?. Biostatistics, 14(1), 129-143.
 #'
 #' @export
 beam.select <- function(object, thres = 0.1, method = "BH", return.only = c(object@return.only,'adj')){
@@ -33,46 +39,36 @@ beam.select <- function(object, thres = 0.1, method = "BH", return.only = c(obje
   #              PREPROCESSING              #
   ###########################################
 
-  # Check arguments
-  if(!is.numeric(thres)){
-    stop("thres must be a numeric")
-  }else{
-    if((thres<=0) | (thres>=1)){
-      stop("thres must be greater than 0 and lower than 1")
-    }
-  }
-  if(!is.null(method)){
-    if(is.character(method)){
-      if(!method%in%c("holm", "bonferroni", "BH", "BY", "HC")){
-        stop("method is not recognized")
-      }
-    }else{
-      stop("method must be a character")
-    }
-  }
-
-  if(is.character(return.only)){
-    if(length(return.only)%in%c(1:4)){
-      ind.return.only <- return.only%in%c("cor", "BF", "prob","adj")
-      if(any(!ind.return.only)){
-        stop("return.only contains characters that are not recognized")
-      }
-    }else{
-      stop("return.only must contain at least 1 element and 4 elements at most")
-    }
-  }else{
-    if(!is.null(return.only)){
-      stop("return.only must be a character")
-    }
-  }
-
-  if(any(c("cor", "BF", "prob") %in% setdiff(return.only, object@return.only))){
-    stop("Some of the requested information is not available in the input beam object")
-  }
+  # Check input argument object
+  assert_that(inherits(object, "beam"))
+  
+  # Check input argument thres
+  assert_that(is.numeric(thres))
+  assert_that(not_empty(thres))
+  assert_that(noNA(thres))
+  assert_that(is.finite(thres), msg="thres is not finite")
+  assert_that((thres>0) & (thres<1), msg="thres must be between 0 and 1")
+  
+  # Check input argument method
+  assert_that(is.character(method))
+  assert_that(not_empty(method))
+  assert_that(length(method)==1)
+  assert_that(noNA(method))
+  assert_that(method%in%c("holm", "bonferroni", "BH", "BY", "HC"), msg="method is not recognized")
+  
+  # Check input argument return.only
+  assert_that(is.character(return.only))
+  assert_that(not_empty(return.only))
+  assert_that(noNA(return.only))
+  return.only <- unique(return.only)
+  labs <- c("cor", "BF", "prob", "adj")
+  assert_that(all(return.only%in%labs), msg="return.only is not recognized")
+  ind <- labs %in% setdiff(return.only, object@return.only)
+  assert_that(any(ind), msg=paste0(paste0(labs[which(ind)], collapse=", "), " not available in input beam object"))
 
   # get the data.frame with marginal and/or conditional estimations
   df <- object@table
-  p <- object@dimX[2]  # get number of covariates
+  p <- object@dimX[2]
   df.cols <- colnames(df)
 
   ###########################################
@@ -97,12 +93,13 @@ beam.select <- function(object, thres = 0.1, method = "BH", return.only = c(obje
     }
 
     if('adj' %in% return.only){ # add a column for adjusted probabilities
-      tableM <- cbind(df[marg.cols], vector(mode = 'numeric', length = p*(p-1)/2))
+      tableM <- as.data.frame(df[, marg.cols])
+      tableM <- cbind(tableM, vector(mode = 'numeric', length = p*(p-1)/2))
       names(tableM)[length(names(tableM))] <- paste0("m_tail_prob_", ifelse(method == 'blfdr',
                                                                             substr(method, 1, 5),
                                                                             substr(method, 1, 4))) # modify last column name
     }else{
-      tableM <- df[marg.cols] # without comma to preserve the data.frame structure even if one single column is selected
+      tableM <- as.data.frame(df[, marg.cols])
     }
 
     if(!('m_tail_prob' %in% df.cols)){
@@ -111,16 +108,16 @@ beam.select <- function(object, thres = 0.1, method = "BH", return.only = c(obje
 
     if(method=="HC"){
 
-      HCthres <- hc.thresh(df$m_tail_prob, alpha0=1, plot=FALSE)
+      HCthres <- hc.thresh(df[,"m_tail_prob"], alpha0=1, plot=FALSE)
       if('adj' %in% return.only){
-        tableM <- tableM[df$m_tail_prob < HCthres, -ncol(tableM)]
+        tableM <- tableM[df[,"m_tail_prob"] < HCthres, -ncol(tableM)]
       }else{
-        tableM <- tableM[df$m_tail_prob < HCthres, , drop = FALSE ]
+        tableM <- tableM[df[,"m_tail_prob"] < HCthres, , drop = FALSE ]
       }
 
     }else{
 
-      tailAdj <- p.adjust(df$m_tail_prob, method=method)
+      tailAdj <- p.adjust(df[,"m_tail_prob"], method=method)
       if('adj' %in% return.only){
         tableM[, ncol(tableM)] <- tailAdj
       }
@@ -153,12 +150,13 @@ beam.select <- function(object, thres = 0.1, method = "BH", return.only = c(obje
     }
 
     if('adj' %in% return.only){ # add a column for adjusted probabilities
-      tableC <- cbind(df[cond.cols], vector(mode = 'numeric', length = p*(p-1)/2))
+      tableC <- as.data.frame(df[, cond.cols])
+      tableC <- cbind(tableC, vector(mode = 'numeric', length = p*(p-1)/2))
       names(tableC)[length(names(tableC))] <- paste0("p_tail_prob_", ifelse(method == 'blfdr',
                                                                             substr(method, 1, 5),
                                                                             substr(method, 1, 4))) # modify last column name
     }else{
-      tableC <- df[cond.cols]
+      tableC <- as.data.frame(df[,cond.cols])
     }
 
     if(!('p_tail_prob' %in% df.cols)){
@@ -167,16 +165,16 @@ beam.select <- function(object, thres = 0.1, method = "BH", return.only = c(obje
 
     if(method=="HC"){
 
-      HCthres <- hc.thresh(df$p_tail_prob, alpha0=1, plot=FALSE)
+      HCthres <- hc.thresh(df[,"p_tail_prob"], alpha0=1, plot=FALSE)
       if('adj' %in% return.only){  # put values in column if requested in output
-        tableC <- tableC[df$p_tail_prob < HCthres, -ncol(tableC)]
+        tableC <- tableC[df[,"p_tail_prob"] < HCthres, -ncol(tableC)]
       }else{
-        tableC <- tableC[df$p_tail_prob < HCthres,  , drop = FALSE]
+        tableC <- tableC[df[,"p_tail_prob"] < HCthres,  , drop = FALSE]
       }
 
     }else{
 
-      tailAdj <- p.adjust(df$p_tail_prob, method=method)
+      tailAdj <- p.adjust(df[,"p_tail_prob"], method=method)
       if('adj' %in% return.only){  # put values in column if requested in output
         tableC[, ncol(tableC)] <- tailAdj
       }
